// Launcher for Darwinia.
// Leander Hambley
// 18/06/05
/////////////////////////

import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;

class Launcher extends JFrame
{
	private JTextField serverAddress1, serverAddress2, serverAddress3,
			textLanguage, textSpeed,
			soundLibrary, soundMixFreq, soundMasterVolume, soundChannels,
			soundMemoryUsage, soundBufferSize, soundDSP,
			screenWidth, screenHeight, screenZDepth, screenColourDepth, screenRefresh,
			renderLandscapeDetail, renderWaterDetail, renderBuildingDetail, 
				renderEntityDetail, renderCloudDetail, renderPixelShader,
			controlUp, controlDown, controlLeft, controlBackwards, controlRight, 
				controlForwards, controlChatLog, controlZoom, controlDeselect,
			controlMouseButtons, executablePath;
	private JCheckBox bypassNetwork, iAmAServer, helpEnabled, 
			soundHW3D, soundSwapStereo,
			screenWindowed, modSystemEnabled, advanced;
	private JComboBox bootLoader, userProfile, mod;
	private JFileChooser executable;
	private Container c;
	private SpringLayout layout;
	private JButton load, save, backup, choose;
	private BufferedReader oldPrefs, fileInput;
	private BufferedWriter newPrefs, backupWriter;
	private File execute;

	public Launcher()
	{
		super("Rubbish looking Darwinia Launcher!");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		c = this.getContentPane();
		layout = new SpringLayout();
		c.setLayout(layout);
		fileInput = null;

		try {
			File prefsTxt = new File("preferences.txt");
			FileReader fr = new FileReader(prefsTxt);
			fileInput = new BufferedReader(fr);		
		} catch (FileNotFoundException fnfe) {
			System.err.println("Preferences file not found: " + fnfe);
			System.err.println("- Please run launcher from Darwinia root directory," +
								"\n    where the preferences.txt file can be found.");
			return;
		}

		setUpComponents(fileInput);
		drawGui();

		this.setSize(800,600);
		this.setVisible(true);
	}

	public void load()
	{
		System.out.println("Loading Darwinia.");
		try {
			if (oldPrefs != null)
			{
				oldPrefs.close();
			}
			if (newPrefs != null)
			{
				newPrefs.close();
			} 
			if (fileInput != null)
			{
				fileInput.close();
			}
		} catch (IOException ioe) {
			System.out.println("Error closing read/write streams: " + ioe);
			return;
		}

		if (executablePath.getText() == null)
		{
			System.out.println("Error: null executable path.");
			JOptionPane.showMessageDialog(this, "Error: null executable path.");
			return;
		}

		String exec = executablePath.getText();
		String exe = exec.substring(executablePath.getText().length()-12,executablePath.getText().length());

		if (System.getProperty("os.name").toLowerCase().equals("windows xp"))
		{
			System.out.println("Windows XP found as OS");
			System.out.println("- Using Darwinia.exe as required filename");

			if (!(exe.toLowerCase().equals("darwinia.exe")))
			{
				System.out.println("WARNING: Unknown executable. Please select the Darwinia.exe executable only.");
				JOptionPane.showMessageDialog(this, "WARNING: Unknown executable. Please select the Darwinia.exe executable only.");
				return;
			}
		} else if (System.getProperty("os.name").toLowerCase().substring(0,3).equals("mac"))
		{
			System.out.println("Mac something found as OS");
			System.out.println("- Using Darwinia.app as required filename");
			if (!(exe.toLowerCase().equals("darwinia.app")))
			{
				System.out.println("WARNING: Unknown executable. Please select the Darwinia.app executable only.");
				JOptionPane.showMessageDialog(this, "WARNING: Unknown executable. Please select the Darwinia.app executable only.");
				return;
			}
		}

		try {
			Process darwinia = Runtime.getRuntime().exec(exec);
		} catch (Exception e) {
			System.err.println("Error running Darwinia: " + e);
		}

		System.out.println("- Done.");
	}

	public void backUp()
	{
		System.out.println("Backing up old preferences.");
		oldPrefs = null;
		backup = null;

		try {
			oldPrefs = new BufferedReader(new FileReader(new File("preferences.txt")));
		} catch (Exception fnfe) {	
			System.err.println("- Failure in opening old preferences file. Exiting.");
			return;
		}

		try {
			java.sql.Date now = new java.sql.Date(System.currentTimeMillis());
			String date = now.toString();
			backupWriter = new BufferedWriter(new FileWriter(new File("preferences-".concat(date).concat(".txt"))));
		} catch (Exception fnfe) {	
			System.err.println("- Failure in opening new backup preferences file. Exiting.");
			return;
		}

		String input = "";

		try {
			while ((input = oldPrefs.readLine()) != null)
			{
				backupWriter.write(input, 0, input.length());
				backupWriter.newLine();
				backupWriter.flush();
			}
			oldPrefs.close();
			backupWriter.close();
		} catch (Exception e) {
			System.out.println("Error backing up preferences: " + e);
		}

		System.out.println("- Done.");
	}

	public void save()
	{
		System.out.println("Saving new preferences.");
		oldPrefs = null;
		newPrefs = null;

		try {
			oldPrefs = new BufferedReader(new FileReader(new File("preferences.txt")));
		} catch (Exception fnfe) {	
			System.err.println("- Failure in opening old preferences file. Exiting.");
			return;
		}

		try {
			newPrefs = new BufferedWriter(new FileWriter(new File("NEWpreferences.txt")));
		} catch (Exception fnfe) {	
			System.err.println("- Failure in opening new temporary preferences file. Exiting.");
			return;
		}

		String input = "";

		try {
			while ((input = oldPrefs.readLine()) != null)
			{
				if (input.equals(""))
				{
					newPrefs.newLine();
					newPrefs.flush();
				} else {
					int counter = 1;
					String read = input.substring(counter-1, counter);
					while(! ( read.substring(read.length()-1, read.length()).equals("=") ) )
					{
						read = input.substring(0, counter);
						counter++;
					}


					if (read.toLowerCase().equals("ServerAddress =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("BypassNetwork =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("IAmAServer =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("TextLanguage =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("TextSpeed =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("HelpEnabled =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundLibrary =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundMixFreq =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundMasterVolume =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundChannels =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundHW3D =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundSwapStereo =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundMemoryUsage =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundBufferSize =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("SoundDSP =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ScreenWidth =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ScreenHeight =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ScreenWindowed =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ScreenZDepth =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ScreenColourDepth =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ScreenRefresh =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("RenderLandscapeDetail =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("RenderWaterDetail =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("RenderBuildingDetail =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("RenderEntityDetail =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("RenderCloudDetail =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("RenderPixelShader =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlUp =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlDown =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlLeft =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlBackwards =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlRight =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlForwards =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlChatLog =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlZoom =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlDeselect =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ControlMouseButtons =".toLowerCase())) 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("BootLoader =".toLowerCase())) 
					{
						String temp = "BootLoader = " + bootLoader.getSelectedItem();
						newPrefs.write(temp, 0, temp.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("UserProfile =".toLowerCase()))
					{
						String temp = "UserProfile = " + userProfile.getSelectedItem();
						newPrefs.write(temp, 0, temp.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("ModSystemEnabled =".toLowerCase())) 
					{
						int x = 0;
						if (modSystemEnabled.isSelected())
						{
							x = 1;
						}
						String temp = "ModSystemEnabled = " + x;
						newPrefs.write(temp, 0, temp.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else if (read.toLowerCase().equals("Mod = ".toLowerCase())) 
					{
						String temp = "Mod = " + mod.getSelectedItem();
						newPrefs.write(temp, 0, temp.length());
						newPrefs.newLine();
						newPrefs.flush();
					} else 
					{
						newPrefs.write(input, 0, input.length());
						newPrefs.newLine();
						newPrefs.flush();
					}
				}
			}
			oldPrefs.close();
			newPrefs.close();

		} catch (Exception e) {
			System.out.println("Error saving preferences: " + e);
			return;
		}

		try {
			oldPrefs = new BufferedReader(new FileReader(new File("NEWpreferences.txt")));
		} catch (Exception fnfe) {	
			System.err.println("- Failure in opening old preferences file. Exiting.");
			return;
		}

		try {
			newPrefs = new BufferedWriter(new FileWriter(new File("preferences.txt")));
		} catch (Exception fnfe) {	
			System.err.println("- Failure in opening new temporary preferences file. Exiting.");
			return;
		}

		try {
			while ((input = oldPrefs.readLine()) != null)
			{
				newPrefs.write(input, 0, input.length());
				newPrefs.newLine();
				newPrefs.flush();
			}
			oldPrefs.close();
			newPrefs.close();
		} catch (Exception e) {
			System.out.println("Error saving preferences: " + e);
			return;
		}

		System.out.println("- Done.");
	}

	public void choose()
	{
		executable.showOpenDialog(this);
		execute = executable.getSelectedFile();

		executablePath.setText(execute.getPath());
	}

	public void drawGui()
	{
		JLabel advancedLabel = new JLabel("Advanced");
		advancedLabel.setToolTipText("Shows advanced options");
		advancedLabel.setEnabled(false);
		advanced.setToolTipText("Shows advanced options");
		
		c.add(advancedLabel);
		layout.putConstraint(SpringLayout.WEST, advancedLabel, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.NORTH, advancedLabel, 25, SpringLayout.NORTH, c);

		c.add(advanced);
		layout.putConstraint(SpringLayout.WEST, advanced, 125, SpringLayout.WEST, advancedLabel);
		layout.putConstraint(SpringLayout.NORTH, advanced, 0, SpringLayout.NORTH, advancedLabel);

		JLabel bootLoaderLabel = new JLabel("Bootloader");
		bootLoaderLabel.setToolTipText("Which bootloader Darwinia will load with");
		bootLoader.setToolTipText("Which bootloader Darwinia will load with");

		c.add(bootLoaderLabel);
		layout.putConstraint(SpringLayout.WEST, bootLoaderLabel, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.NORTH, bootLoaderLabel, 75, SpringLayout.NORTH, c);

		c.add(bootLoader);
		layout.putConstraint(SpringLayout.WEST, bootLoader, 125, SpringLayout.WEST, bootLoaderLabel);
		layout.putConstraint(SpringLayout.NORTH, bootLoader, 0, SpringLayout.NORTH, bootLoaderLabel);

		JLabel userLabel = new JLabel("User");
		userLabel.setToolTipText("Which user Darwinia will load with");
		userProfile.setToolTipText("Which user Darwinia will load with");

		c.add(userLabel);
		layout.putConstraint(SpringLayout.WEST, userLabel, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.NORTH, userLabel, 125, SpringLayout.NORTH, c);

		c.add(userProfile);
		layout.putConstraint(SpringLayout.WEST, userProfile, 125, SpringLayout.WEST, userLabel);
		layout.putConstraint(SpringLayout.NORTH, userProfile, 0, SpringLayout.NORTH, userLabel);

		JLabel modSystemEnabledLabel = new JLabel("ModSystemEnabled");
		modSystemEnabledLabel.setToolTipText("Is the map editor available in Darwinia");
		modSystemEnabled.setToolTipText("Is the map editor available in Darwinia");

		c.add(modSystemEnabledLabel);
		layout.putConstraint(SpringLayout.WEST, modSystemEnabledLabel, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.NORTH, modSystemEnabledLabel, 175, SpringLayout.NORTH, c);

		c.add(modSystemEnabled);
		layout.putConstraint(SpringLayout.WEST, modSystemEnabled, 125, SpringLayout.WEST, modSystemEnabledLabel);
		layout.putConstraint(SpringLayout.NORTH, modSystemEnabled, 0, SpringLayout.NORTH, modSystemEnabledLabel);

		JLabel modLabel = new JLabel("Mod");
		modLabel.setToolTipText("Which mod Darwinia will load with");
		mod.setToolTipText("Which mod Darwinia will load with");

		c.add(modLabel);
		layout.putConstraint(SpringLayout.WEST, modLabel, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.NORTH, modLabel, 225, SpringLayout.NORTH, c);

		c.add(mod);
		layout.putConstraint(SpringLayout.WEST, mod, 125, SpringLayout.WEST, modLabel);
		layout.putConstraint(SpringLayout.NORTH, mod, 0, SpringLayout.NORTH, modLabel);

		load.setToolTipText("Which mod Darwinia will load with");

		c.add(load);
		layout.putConstraint(SpringLayout.WEST, load, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.SOUTH, load, -50, SpringLayout.SOUTH, c);

		save.setToolTipText("Save you're preferences ");

		c.add(save);
		layout.putConstraint(SpringLayout.WEST, save, 25, SpringLayout.EAST, load);
		layout.putConstraint(SpringLayout.SOUTH, save, -50, SpringLayout.SOUTH, c);

		backup.setToolTipText("Save you're preferences ");

		c.add(backup);
		layout.putConstraint(SpringLayout.WEST, backup, 25, SpringLayout.EAST, save);
		layout.putConstraint(SpringLayout.SOUTH, backup, -50, SpringLayout.SOUTH, c);

		JLabel executableLabel = new JLabel("Your Darwinia executable path:");

		executableLabel.setToolTipText("Path of your Darwinia exectuable");
		executablePath.setToolTipText("Path of your Darwinia exectuable");
		choose.setToolTipText("Search for your Darwinia executable");

		c.add(executablePath);
		layout.putConstraint(SpringLayout.WEST, executablePath, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.SOUTH, executablePath, -100, SpringLayout.SOUTH, c);

		c.add(executableLabel);
		layout.putConstraint(SpringLayout.WEST, executableLabel, 25, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.SOUTH, executableLabel, -120, SpringLayout.SOUTH, c);

		c.add(choose);
		layout.putConstraint(SpringLayout.WEST, choose, 275, SpringLayout.WEST, c);
		layout.putConstraint(SpringLayout.SOUTH, choose, -100, SpringLayout.SOUTH, c);
	}

	public void setUpComponents(BufferedReader fileIn)
	{
		try {
			String input = fileIn.readLine();
			String read = null;

			while(input != null)
			{
				if (input.equals(""))
				{
					// A new line is found.
					// Don't try and do stuff with it.
				} else {
					int counter = 1;
					read = input.substring(counter-1, counter);
					while(! ( read.substring(read.length()-1, read.length()).equals("=") ) )
					{
						read = input.substring(0, counter);
						counter++;
					}

					System.out.println(read);
					if (read.toLowerCase().equals("ServerAddress =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("BypassNetwork =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("IAmAServer =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("TextLanguage =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("TextSpeed =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("HelpEnabled =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundLibrary =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundMixFreq =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundMasterVolume =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundChannels =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundHW3D =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundSwapStereo =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundMemoryUsage =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundBufferSize =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("SoundDSP =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ScreenWidth =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ScreenHeight =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ScreenWindowed =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ScreenZDepth =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ScreenColourDepth =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ScreenRefresh =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("RenderLandscapeDetail =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("RenderWaterDetail =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("RenderBuildingDetail =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("RenderEntityDetail =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("RenderCloudDetail =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("RenderPixelShader =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlUp =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlDown =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlLeft =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlBackwards =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlRight =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlForwards =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlChatLog =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlZoom =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlDeselect =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("ControlMouseButtons =".toLowerCase())) 
					{
					} else if (read.toLowerCase().equals("BootLoader =".toLowerCase())) 
					{
						System.out.println("- Line is \"BootLoader =\"");
						String bootloaders[] = {"none", "firsttime", "random", "spectrum", "matrix", "gameoflife", 
												"gameoflife_glow", "soul", "raytrace", "fodder", "credits"};
						String currentLoader = input.substring(read.length(), input.length()).trim();
						int selected = -1;
						for (int i = 0; i < bootloaders.length; i++)
						{
							if (currentLoader.toLowerCase().equals(bootloaders[i].toLowerCase().trim()))
							{
								System.out.println("- Current bootloader is:" + bootloaders[i]);
								selected = i;
								i = bootloaders.length;
							}
						}
						bootLoader = new JComboBox((Object[])bootloaders);
						bootLoader.setSelectedIndex(selected);
					} else if (read.toLowerCase().equals("UserProfile =".toLowerCase()))
					{
						System.out.println("- Line is \"UserProfile =\"");
						String user = input.substring(read.length(),input.length());
						System.out.println("- Current user is:" + user);
						int selected = -1;
						File userDir = new File(".\\users");
						String dirs[] = userDir.list();
						ArrayList users = new ArrayList();
						if (dirs == null)
						{
							System.out.println("- Error: User folder listing is null.");
						} else {
							for (int i = 0; i < dirs.length; i++)
							{
								File tempFile = new File(".\\users\\" + dirs[i]);
								if (user.toLowerCase().trim().equals(dirs[i].toLowerCase().trim()))
								{
									System.out.println("- Selected user is:" + dirs[i]);
									selected = i;
								}
								if (tempFile.isDirectory())
								{
									users.add(dirs[i]);
								}
							}
						}
						Object[] userArray = users.toArray();
						userProfile = new JComboBox(userArray);
						userProfile.setSelectedIndex(selected);
					} else if (read.toLowerCase().equals("ModSystemEnabled =".toLowerCase())) 
					{
						System.out.println("- Line is \"ModSystemEnabled =\"");
						String number = input.substring(read.length(),input.length());
						number = number.trim();
						int num = -1;
						try {
							num = Integer.parseInt(number);
						} catch (NumberFormatException nfe) {
							System.err.println("- Error reading ModSystemEnabled value. Exiting");
							return;
						}
						modSystemEnabled = new JCheckBox();
						System.out.println("- ModSystemEnabled:" + !(num < 1));
						modSystemEnabled.setSelected(!(num < 1));
					} else if (read.toLowerCase().equals("Mod =".toLowerCase())) 
					{
						System.out.println("- Line is \"Mod =\"");
						String currentMod = input.substring(read.length(),input.length());
						System.out.println("- Current mod is:" + currentMod);
						int selected = -1;
						File modDir = new File(".\\mods");
						String dirs[] = modDir.list();
						ArrayList mods = new ArrayList();
						if (dirs == null)
						{
							System.err.println("Error: Mod folder listing is null.");
						} else {
							String tempDirs[] = dirs;
							dirs = new String[dirs.length+1];
							dirs[0] = "none";
							for (int i = 0; i < tempDirs.length; i++)
							{
								dirs[i+1] = tempDirs[i];
							}

							for (int i = 0; i < dirs.length; i++)
							{
								File tempFile = new File(".\\mods\\" + dirs[i]);
								if (currentMod.toLowerCase().trim().equals(dirs[i].toLowerCase().trim()))
								{
									System.out.println("- Selected mod is:" + dirs[i]);
									selected = i;
								}
								if (tempFile.isDirectory())
								{
									mods.add(dirs[i]);
								} else if (dirs[i].equals("none"))
								{
									mods.add(dirs[i]);
								}
							}
						}
						Object[] modArray = mods.toArray();
						mod = new JComboBox(modArray);
						mod.setSelectedIndex(selected);
					} 
				}
				input = fileIn.readLine();
			}
			advanced = new JCheckBox();
			advanced.setEnabled(false);

			load = new JButton("Load Darwinia");
			save = new JButton("Save Preferences");
			backup = new JButton("Backup");
			choose = new JButton("Search");

			load.addActionListener(new LoadHandler());
			save.addActionListener(new SaveHandler());
			backup.addActionListener(new BackupHandler());
			choose.addActionListener(new ChooserHandler());

			executablePath = new JTextField(20);
			executable = new JFileChooser();
		} catch (IOException ioe) {
			System.err.println("Reading preferences file failed: " + ioe);
			System.err.println("Exiting");
			System.exit(0);
		}
		System.out.println("Done");
	}

	public static void main(String argvs[])
	{
		Launcher l = new Launcher();
	}

	class SaveHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			save();
		}

	}

	class BackupHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			backUp();
		}

	}

	class LoadHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			load();
		}

	}

	class ChooserHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			choose();
		}
	}
}